<?php
namespace Teknisa\CTB\Factories;

use Doctrine\ORM\EntityManager;

class Ctb10000LancamentosContabeisFactory {

    protected $connection;

    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
    }

    /*
     * Função para deletar um lançamento, dados os parâmetros.
     */
    public function deleteLanccont($params) {
        // Executa a query para remover rateios associados.
        $this->deleteRateiosLancamento($params);

        // Query a ser executada.
        $sql = "DELETE FROM LANCCONT WHERE CDEMPRESA = :CDEMPRESA AND NRLANCCONT = :NRLANCCONT AND NRORG = :NRORG";

        // Prepara o statement da query para execução.
        $statement = $this->connection->prepare($sql);

        // Executa o statement com os paràmetros passados.
        $statement->execute($params);
    }

    /*
     * Função para deletar uma partida de um lançamento, dados os parâmetros.
     */
    public function deletePartida($params) {
        // Executa a query para remover rateios associados.
        $this->deleteRateiosPartida($params);

        // Query a ser executada.
        $sql = "DELETE FROM LANCCONT WHERE CDEMPRESA = :CDEMPRESA AND NRLANCCONT = :NRLANCCONT AND NRSEQLANC = :NRSEQLANC AND NRORG = :NRORG";

        // Prepara o statement da query para execução.
        $statement = $this->connection->prepare($sql);

        // Executa o statement com os paràmetros passados.
        $statement->execute($params);
    }

    /*
     * Função para deletar os rateios de um lançamento, dados os parâmetros.
     */
    public function deleteRateiosLancamento($params) {
        // Query a ser executada.
        $sql = "DELETE FROM LANCCCUSTO WHERE CDEMPRESA = :CDEMPRESA AND NRLANCCONT = :NRLANCCONT AND NRORG = :NRORG";

        // Prepara o statement da query para execução.
        $statement = $this->connection->prepare($sql);

        // Executa o statement com os paràmetros passados.
        $statement->execute($params);
    }

    /*
     * Função para deletar os rateios de uma partida de um lançamento, dados os parâmetros.
     */
    public function deleteRateiosPartida($params) {
        // Query a ser executada.
        $sql = "DELETE FROM LANCCCUSTO WHERE CDEMPRESA = :CDEMPRESA AND NRLANCCONT = :NRLANCCONT AND NRSEQLANC = :NRSEQLANC AND NRORG = :NRORG";

        // Prepara o statement da query para execução.
        $statement = $this->connection->prepare($sql);

        // Executa o statement com os paràmetros passados.
        $statement->execute($params);
    }

    /*
     * Função para retornar os dados das partidas de um lançamento, dados os parâmetros.
     */
    public function getNextSequencialLancamento($params) {
        // Query a ser executada.
        $sql = "SELECT MAX(NRSEQLANC) AS NRSEQLANC FROM LANCCONT WHERE CDEMPRESA = :CDEMPRESA AND NRLANCCONT = :NRLANCCONT AND NRORG = :NRORG";

        // Prepara o statement da query para execução.
        $statement = $this->connection->prepare($sql);

        // Executa o statement com os paràmetros passados.
        $statement->execute($params);

        // Executa um fetch no primeiro dado, retornando-o.
        return $statement->fetch(\PDO::FETCH_ASSOC);
    }

    /*
     * Função para retornar os dados das partidas de um lançamento, dados os parâmetros.
     */
    public function getPartidas($params) {
        // Query a ser executada.
        $sql = "SELECT L.CDEMPRESA, L.NRLANCCONT, L.NRSEQLANC, C.CDCONTCTBL, C.NMCONTCTBL, TO_CHAR(L.DTLANCCONT, 'DD/MM/YYYY') DTLANCCONT,
               TO_NUMBER(DECODE(L.VRLANCAMENT, ABS(L.VRLANCAMENT), L.VRLANCAMENT, NULL)) CREDITO,
               TO_NUMBER(DECODE(L.VRLANCAMENT, ABS(L.VRLANCAMENT), NULL, ABS(L.VRLANCAMENT))) DEBITO,
               DECODE(L.VRLANCAMENT, ABS(L.VRLANCAMENT), 'CR', 'DB') IDDEBCRE,
               ABS(L.VRLANCAMENT) VALORABS,
               L.VRLANCAMENT, L.CDDOCUCONT, L.DSHISTLANC, C.IDTIPOCOME IDLANCCOME, L.CDCONTCOME, C.IDCONTCCUS,
               L.IDLANCINTE, C.CDCONTREDU, L.IDTIPOLANC, L.CDHISTPADR, L.NRORG
         FROM LANCCONT L, CONTCTBL C
        WHERE L.CDCONTCTBL = C.CDCONTCTBL
          AND L.NRORG = C.NRORG
          AND L.NRLANCCONT = :NRLANCCONT
          AND L.CDEMPRESA = :CDEMPRESA
          AND L.NRORG = :NRORG
        ORDER BY L.NRLANCCONT, L.NRSEQLANC";

        // Prepara o statement da query para execução.
        $statement = $this->connection->prepare($sql);

        // Executa o statement com os paràmetros passados.
        $statement->execute($params);

        // Executa um fetch em todos os dados, retornando-os.
        return $statement->fetchAll();
    }

    /*
     * Função para retornar os dados dos rateios de um lançamento, dados os parâmetros.
     */
    public function getRateios($params) {
        // Query a ser executada.
        $sql = "SELECT L.CDEMPRESA, L.NRLANCCONT, L.NRSEQLANC, L.CDFILIAL, F.NMFILIAL, L.CDCENTCUST,
               C.NMCENTCUST, L.DTCOMPCUSTO, L.VRLANCCUST,
               TO_NUMBER(DECODE(L.VRLANCCUST, ABS(L.VRLANCCUST), L.VRLANCCUST, NULL)) CREDITO,
               TO_NUMBER(DECODE(L.VRLANCCUST, ABS(L.VRLANCCUST), NULL, ABS(L.VRLANCCUST))) DEBITO,
               DECODE(L.VRLANCCUST, ABS(L.VRLANCCUST), 'CR', 'DB') IDDEBCRE,
               ABS(L.VRLANCCUST) VALORABS
          FROM LANCCCUSTO L, FILIAL F, CENTCUST C
         WHERE L.CDEMPRESA = :CDEMPRESA
           AND L.NRLANCCONT = :NRLANCCONT
           AND L.NRORG = :NRORG
           AND L.CDFILIAL = F.CDFILIAL
           AND L.NRORG = F.NRORG
           AND L.CDCENTCUST = C.CDCENTCUST
           AND L.NRORG = C.NRORG
         ORDER BY L.CDEMPRESA, L.NRLANCCONT, L.NRSEQLANC, L.CDFILIAL, L.CDCENTCUST";

        // Prepara o statement da query para execução.
        $statement = $this->connection->prepare($sql);

        // Executa o statement com os paràmetros passados.
        $statement->execute($params);

        // Executa um fetch em todos os dados, retornando-os
        return $statement->fetchAll();
    }
    
    /**
     * Rastreio Contabil
     * @param $cdfilial
     * @param $cdempresa
     * @return array
     */
    public function rastreioContabil(
        $cdempresa, $nrlanccont
    ){
        $params = array();
        $params['P_CDEMPRESA']    = $cdempresa;
        $params['P_NRLANCCONT']   = $nrlanccont;
        
                       
        $sql =  "SELECT DISTINCT
                   DADOS.*
                FROM
                   (SELECT 
                       NF.DTENTRSAID AS DATA,
                       NF.CDFILIRECE AS FILIAL,
                       NF.NRNOTAFISC AS NRDOCU,
                       'DFE' AS ORIGEM,
                       NF.VRNOTAFISC AS VRDOCU,
                       NF.CDEMPRESA,
					   EM.NMEMPRESA,
                       NF.CDCLIENTE,
                       NF.CDFORNECED,
                       LANC.NRLANCCONT
                    FROM
                       CONTCTBL CONT,
                       NOTAFISCAL NF,
                       LANCCONT LANC,
					   EMPRESA EM
                    WHERE
                       (LANC.CDEMPRESA  = :P_CDEMPRESA)                                                   AND
                       (LANC.NRLANCCONT = :P_NRLANCCONT)                                                  AND
                       (LANC.IDLANCINTE = 'I')                                                          AND
                       (LANC.CDCONTCTBL = CONT.CDCONTCTBL)                                              AND
                       (LANC.CDEMPRESA  = NF.CDEMPRESA)                                                 AND
                       (LANC.NRLANCCONT = NF.NRLANCTBDOC)                                               AND
                       (NF.IDENTRSAIDOP = 'E')                                                          AND
                       (NF.IDSISTEMA = 'EST' OR NF.IDSISTEMA = 'DFE' OR NF.IDSISTEMA IS NULL) AND
					   (NF.CDEMPRESA = EM.CDEMPRESA)
                
                    UNION ALL
                
                    SELECT
                       NF.DTENTRSAID AS DATA,
                       NF.CDFILIEMIT AS FILIAL,
                       NF.NRNOTAFISC AS NRDOCU,
                       'DFS' AS ORIGEM,
                       NF.VRNOTAFISC AS VRDOCU,
                       NF.CDEMPRESA,
					   EM.NMEMPRESA,
                       NF.CDCLIENTE,
                       NF.CDFORNECED,
                       LANC.NRLANCCONT
                    FROM
                       CONTCTBL CONT,
                       NOTAFISCAL NF,
                       LANCCONT LANC,
					   EMPRESA EM
                    WHERE
                       (LANC.CDEMPRESA  = :P_CDEMPRESA)                                                   AND
                       (LANC.NRLANCCONT = :P_NRLANCCONT)                                                  AND
                       (LANC.IDLANCINTE = 'I')                                                          AND
                       (LANC.CDCONTCTBL = CONT.CDCONTCTBL)                                              AND
                       (LANC.CDEMPRESA  = NF.CDEMPRESA)                                                 AND
                       (LANC.NRLANCCONT = NF.NRLANCTBDOC)                                               AND
                       (NF.IDENTRSAIDOP = 'S')                                                          AND
                       (NF.IDSISTEMA = 'EST' OR NF.IDSISTEMA = 'DFS' OR NF.IDSISTEMA = 'FAI' OR 
                        NF.IDSISTEMA = 'FAS' OR NF.IDSISTEMA = 'FAC' OR NF.IDSISTEMA IS NULL) AND
					   (NF.CDEMPRESA = EM.CDEMPRESA)
                
                    UNION ALL
                
                    SELECT
                       NF.DTENTRSAID AS DATA,
                       NF.CDFILIEMIT AS FILIAL,
                       NF.NRNOTAFISC AS NRDOCU,
                       'PAG' AS ORIGEM,
                       NF.VRNOTAFISC AS VRDOCU,
                       NF.CDEMPRESA,
					   EM.NMEMPRESA,
                       NF.CDCLIENTE,
                       NF.CDFORNECED,
                       LANC.NRLANCCONT
                    FROM
                       CONTCTBL CONT,
                       NOTAFISCAL NF,
                       LANCCONT LANC,
					   EMPRESA EM
                    WHERE
                       (LANC.CDEMPRESA  = :P_CDEMPRESA)                                                   AND
                       (LANC.NRLANCCONT = :P_NRLANCCONT)                                                  AND
                       (LANC.IDLANCINTE = 'I')                                                          AND
                       (LANC.CDCONTCTBL = CONT.CDCONTCTBL)                                              AND
                       (LANC.CDEMPRESA  = NF.CDEMPRESA)                                                 AND
                       (LANC.NRLANCCONT = NF.NRLANCTBDOC)                                               AND
                       (NF.IDSISTEMA = 'PAG') AND
					   (NF.CDEMPRESA = EM.CDEMPRESA)
                
                    UNION ALL
                
                    SELECT
                       NF.DTENTRSAID AS DATA,
                       NF.CDFILIEMIT AS FILIAL,
                       NF.NRNOTAFISC AS NRDOCU,
                       'REC' AS ORIGEM,
                       NF.VRNOTAFISC AS VRDOCU,
                       NF.CDEMPRESA,
					   EM.NMEMPRESA,
                       NF.CDCLIENTE,
                       NF.CDFORNECED,
                       LANC.NRLANCCONT
                    FROM
                       CONTCTBL CONT,
                       NOTAFISCAL NF,
                       LANCCONT LANC,
					   EMPRESA EM
                    WHERE
                       (LANC.CDEMPRESA  = :P_CDEMPRESA)                                                   AND
                       (LANC.NRLANCCONT = :P_NRLANCCONT)                                                  AND
                       (LANC.IDLANCINTE = 'I')                                                          AND
                       (LANC.CDCONTCTBL = CONT.CDCONTCTBL)                                              AND
                       (LANC.CDEMPRESA  = NF.CDEMPRESA)                                                 AND
                       (LANC.NRLANCCONT = NF.NRLANCTBDOC)                                               AND
                       (NF.IDSISTEMA = 'REC') AND
					   (NF.CDEMPRESA = EM.CDEMPRESA)
                
                    UNION ALL
                
                    SELECT
                       AJPAG.DTAJUSTEPAG AS DATA,
                       AJPAG.CDFILIAL AS FILIAL,
                       NF.NRNOTAFISC AS NRDOCU,
                       'AJUSTE_PAG' AS ORIGEM,
                       AJPAG.VRAJUSTEPAG AS VRDOCU,
                       NF.CDEMPRESA,
					   EM.NMEMPRESA,
                       NF.CDCLIENTE,
                       NF.CDFORNECED,
                       LANC.NRLANCCONT
                    FROM
                       CONTCTBL CONT,
                       NOTAFISCAL NF,
                       AJCTPAGAR AJPAG,
                       LANCCONT LANC,
					   EMPRESA EM
                    WHERE
                       (LANC.CDEMPRESA  = :P_CDEMPRESA)                                                   AND
                       (LANC.NRLANCCONT = :P_NRLANCCONT)                                                  AND
                       (LANC.IDLANCINTE = 'I')                                                          AND
                       (LANC.CDCONTCTBL = CONT.CDCONTCTBL)                                              AND
                       (LANC.CDEMPRESA  = NVL(AJPAG.CDECTBAJPAG,AJPAG.CDEMMFAJCNTP))                    AND
                       (LANC.NRLANCCONT = AJPAG.NRLCTBAJPAG)                                            AND
                       (AJPAG.CDFILIAL   = NF.CDFILIAL)                                                 AND
                       (AJPAG.NRLANCTONF = NF.NRLANCTONF) AND
					   (NF.CDEMPRESA = EM.CDEMPRESA)
                
                    UNION ALL
                
                    SELECT
                       AJREC.DTAJUSTEREC AS DATA,
                       AJREC.CDFILIAL AS FILIAL,
                       NF.NRNOTAFISC AS NRDOCU,
                       'AJUSTE_REC' AS ORIGEM,
                       AJREC.VRAJUSTEREC AS VRDOCU,
                       NF.CDEMPRESA,
					   EM.NMEMPRESA,
                       NF.CDCLIENTE,
                       NF.CDFORNECED,
                       LANC.NRLANCCONT
                    FROM
                       CONTCTBL CONT,
                       NOTAFISCAL NF,
                       AJCTRECEBER AJREC,
                       LANCCONT LANC,
					   EMPRESA EM
                    WHERE
                       (LANC.CDEMPRESA  = :P_CDEMPRESA)                                                   AND
                       (LANC.NRLANCCONT = :P_NRLANCCONT)                                                  AND
                       (LANC.IDLANCINTE = 'I')                                                          AND
                       (LANC.CDCONTCTBL = CONT.CDCONTCTBL)                                              AND
                       (LANC.CDEMPRESA  = NVL(AJREC.CDECTBAJREC, AJREC.CDEMMFAJCNTR))                   AND
                       (LANC.NRLANCCONT = AJREC.NRLCTBAJREC)                                            AND
                       (AJREC.CDFILIAL   = NF.CDFILIAL)                                                 AND
                       (AJREC.NRLANCTONF = NF.NRLANCTONF) AND
					   (NF.CDEMPRESA = EM.CDEMPRESA)
                
                    UNION ALL
                
                    SELECT
                       BPAGA.DTBAIXAPAG AS DATA,
                       BPAGA.CDFILIAL AS FILIAL,
                       BPAGA.NRBAIXAPAG AS NRDOCU,
                       'BAIXA_PAG' AS ORIGEM,
                       BPAGA.VRBAIXAPAG AS VRDOCU,
                       BPAGA.CDEMPRESA,
					   EM.NMEMPRESA,
                       '' AS CDCLIENTE,
                       '' AS CDFORNECED,
                       LANC.NRLANCCONT
                    FROM
                       CONTCTBL CONT,
                       BAIXAPAGA BPAGA,
                       LANCCONT LANC,
					   EMPRESA EM
                    WHERE
                       (LANC.CDEMPRESA  = :P_CDEMPRESA)                                                   AND
                       (LANC.NRLANCCONT = :P_NRLANCCONT)                                                  AND
                       (LANC.IDLANCINTE = 'I')                                                          AND
                       (LANC.CDCONTCTBL = CONT.CDCONTCTBL)                                              AND
                       (LANC.CDEMPRESA  = BPAGA.CDEMPRESA)                                              AND
                       (LANC.NRLANCCONT = BPAGA.NRLCTBBPAG) AND
					   (BPAGA.CDEMPRESA = EM.CDEMPRESA)
                
                    UNION ALL
                
                    SELECT
                       BRECE.DTBAIXAREC AS DATA,
                       BRECE.CDFILIAL AS FILIAL,
                       BRECE.NRBAIXAREC AS NRDOCU,
                       'BAIXA_REC' AS ORIGEM,
                       BRECE.VRBAIXAREC AS VRDOCU,
                       BRECE.CDEMPRESA,
					   EM.NMEMPRESA,
                       '' AS CDCLIENTE,
                       '' AS CDFORNECED,
                       LANC.NRLANCCONT
                    FROM
                       CONTCTBL CONT,
                       BAIXARECEBIDA BRECE,
                       LANCCONT LANC,
					   EMPRESA EM
                    WHERE
                       (LANC.CDEMPRESA  = :P_CDEMPRESA)                                                   AND
                       (LANC.NRLANCCONT = :P_NRLANCCONT)                                                  AND
                       (LANC.IDLANCINTE = 'I')                                                          AND
                       (LANC.CDCONTCTBL = CONT.CDCONTCTBL)                                              AND
                       (LANC.CDEMPRESA  = BRECE.CDEMPRESA)                                              AND
                       (LANC.NRLANCCONT = BRECE.NRLCTBBREC) AND
					   (BRECE.CDEMPRESA = EM.CDEMPRESA)
                
                    UNION ALL
                
                    SELECT
                       MCREB.DTMOVICRED AS DATA,
                       MCREB.CDFILIAL AS FILIAL,
                       MCREB.NRMOVICRED AS NRDOCU,
                       'MOVICRED_BCO' AS ORIGEM,
                       MCREB.VRMOVICRED AS VRDOCU,
                       '' AS CDEMPRESA,
					   '' AS NMEMPRESA,
                       '' AS CDCLIENTE,
                       '' AS CDFORNECED,
                       LANC.NRLANCCONT
                    FROM
                       CONTCTBL CONT,
                       MOVCREDBANC MCREB,
                       LANCCONT LANC
                    WHERE
                       (LANC.CDEMPRESA  = :P_CDEMPRESA)                                                   AND
                       (LANC.NRLANCCONT = :P_NRLANCCONT)                                                  AND
                       (LANC.IDLANCINTE = 'I')                                                          AND
                       (LANC.CDCONTCTBL = CONT.CDCONTCTBL)                                              AND
                       (LANC.CDEMPRESA  = MCREB.CDECTBMCRED)                                            AND
                       (LANC.NRLANCCONT = MCREB.NRLCTBMCRED)
                
                    UNION ALL
                
                    SELECT
                       MDEBB.DTMOVIDEBI AS DATA,
                       MDEBB.CDFILIAL AS FILIAL,
                       MDEBB.NRMOVIDEBI AS NRDOCU,
                       'MOVIDEB_BCO' AS ORIGEM,
                       MDEBB.VRMOVIDEBI AS VRDOCU,
                       '' AS CDEMPRESA,
					   '' AS NMEMPRESA,
                       '' AS CDCLIENTE,
                       '' AS CDFORNECED,
                       LANC.NRLANCCONT
                    FROM
                       CONTCTBL CONT,
                       MOVDEBBANC MDEBB,
                       LANCCONT LANC
                    WHERE
                       (LANC.CDEMPRESA  = :P_CDEMPRESA)                                                   AND
                       (LANC.NRLANCCONT = :P_NRLANCCONT)                                                  AND
                       (LANC.IDLANCINTE = 'I')                                                          AND
                       (LANC.CDCONTCTBL = CONT.CDCONTCTBL)                                              AND
                       (LANC.CDEMPRESA  = MDEBB.CDECTBMDEB)                                             AND
                       (LANC.NRLANCCONT = MDEBB.NRLCTBMDEB)
                
                    UNION ALL
                
                    SELECT
                       TRANB.DTTRANBANC AS DATA,
                       TRANB.CDFILIAL AS FILIAL,
                       TRANB.NRTRANBANC AS NRDOCU,
                       'TRANSF_BCO' AS ORIGEM,
                       TRANB.VRTRANBANC AS VRDOCU,
                       '' AS CDEMPRESA,
					   '' AS NMEMPRESA,
                       '' AS CDCLIENTE,
                       '' AS CDFORNECED,
                       LANC.NRLANCCONT
                    FROM
                       CONTCTBL CONT,
                       TRANSFBANC TRANB,
                       LANCCONT LANC
                    WHERE
                       (LANC.CDEMPRESA  = :P_CDEMPRESA)                                                   AND
                       (LANC.NRLANCCONT = :P_NRLANCCONT)                                                  AND
                       (LANC.IDLANCINTE = 'I')                                                          AND
                       (LANC.CDCONTCTBL = CONT.CDCONTCTBL)                                              AND
                       (LANC.CDEMPRESA  = TRANB.CDEMPRESA)                                              AND
                       (LANC.NRLANCCONT = TRANB.NRLCTBTRAN)
                   ) DADOS
                ORDER BY
                   1,
                   6";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

}