<?php

namespace Teknisa\CTB\Services;

use Teknisa\Model\Contabilidade\Entities\Lanccont;
use Zeedhi\Framework\DTO;
use Teknisa\Libs\Util\DateUtil;
use Zeedhi\Framework\DataSource\DataSet;

class ContopectblEmpresa {

    protected $crudService;
    protected $dataSourceManager;

    public function __construct($crudService, $dataSourceManager,$entityManager) {
        $this->crudService = $crudService;
        $this->dataSourceManager = $dataSourceManager;
        $this->entityManager = $entityManager;
    }

    public function ferificDataSuperior($nrorg, $cdempresa, $dtfinacont) {

        $this->dataSourceName = "saldempr_maiordata";
        $this->parameterBagColumns = array("P_CDEMPRESA", "P_NRORG");

        $filterCriteria = new \Zeedhi\Framework\DataSource\FilterCriteria("saldempr_maiordata");
        $filterCriteria->addCondition('P_CDEMPRESA',$cdempresa);
        $filterCriteria->addCondition('P_NRORG', $nrorg);
        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
        $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
        $data =  $dataSet->getRows();

        $vericado = $this->verificaData($data, $dtfinacont);
        if(!$vericado){
            throw new \Exception("Existe saldo com data superior a informada no final do período contábil.", 1);
        }
    }

    public function setIdsaldempr($row, $dataSourceName, $method, $routePath, $userId){
        if(!isset($row["IDSALDEMPR"])){
            $row["IDSALDEMPR"] = 'M';
        }
        $row  = array($row);
        $request = $this->requestSet($row, $dataSourceName, $method, $routePath, $userId);
        return $request;
    }

    public function checaLancaEmp($nrorg, $cdempresa){

        //$banco = Lanccont::findOneByNrorgAndCdempresa($nrorg, $cdempresa);
        $banco = $this->entityManager->getRepository('Teknisa\Model\Contabilidade\Entities\Lanccont')->findOneBy(
                array('nrorg' => $nrorg, 'cdempresa' => $cdempresa)
        );
        if(is_object($banco)){
            throw new \Exception("Existem lançamentos contábeis cadastrados para empresa informada.", 1);
        }
    }

    public function checaRow($row){
        $dtiniccont = DateUtil::getDataDeString($row["DTINICCONT"].'00:00:00', DateUtil::FORMATO_DATAHORA);
        $dtfinacont = DateUtil::getDataDeString($row["DTFINACONT"].'00:00:00', DateUtil::FORMATO_DATAHORA);
        $dtiniclanc = DateUtil::getDataDeString($row["DTINICLANC"].'00:00:00', DateUtil::FORMATO_DATAHORA);
        $dtfinalanc = DateUtil::getDataDeString($row["DTFINALANC"].'00:00:00', DateUtil::FORMATO_DATAHORA);
        $dtiniintcontdfe = DateUtil::getDataDeString($row["DTINIINTCONTDFE"].'00:00:00', DateUtil::FORMATO_DATAHORA);
        $dtfinintcontdfe = DateUtil::getDataDeString($row["DTFININTCONTDFE"].'00:00:00', DateUtil::FORMATO_DATAHORA);
        $dtiniintcontdfs = DateUtil::getDataDeString($row["DTINIINTCONTDFS"].'00:00:00', DateUtil::FORMATO_DATAHORA);
        $dtfinintcontdfs = DateUtil::getDataDeString($row["DTFININTCONTDFS"].'00:00:00', DateUtil::FORMATO_DATAHORA);
        $dtinicctbl = DateUtil::getDataDeString($row["DTINICCTBL"].'00:00:00', DateUtil::FORMATO_DATAHORA);
        $dtfinacctbl = DateUtil::getDataDeString($row["DTFINACCTBL"].'00:00:00', DateUtil::FORMATO_DATAHORA);
        $dtiniintcontgpa = DateUtil::getDataDeString($row["DTINIINTCONTGPA"].'00:00:00', DateUtil::FORMATO_DATAHORA);
        $dtfinintcontgpa = DateUtil::getDataDeString($row["DTFININTCONTGPA"].'00:00:00', DateUtil::FORMATO_DATAHORA);

        if(!DateUtil::comparaDatas($dtiniccont, $dtfinacont) && !DateUtil::comparaDatas($dtiniccont, $dtfinacont, DateUtil::IGUAL)){
            throw new \Exception("A data inicial do Período Contábil deve ser menor que a data final do Período Contábil.", 1);
        } else if(!DateUtil::comparaDatas($dtiniclanc, $dtfinalanc) && !DateUtil::comparaDatas($dtiniclanc, $dtfinalanc, DateUtil::IGUAL)){
            throw new \Exception("A data inicial do Período de Lançamento deve ser menor que a data final do Período de Lançamento.", 1);
        } else if(!DateUtil::comparaDatas($dtiniintcontdfe, $dtfinintcontdfe) && !DateUtil::comparaDatas($dtiniintcontdfe, $dtfinintcontdfe, DateUtil::IGUAL)){
            throw new \Exception("A data inicial do Doc.Fiscais Entrada deve ser maior que a data final do Doc.Fiscais Entrada.", 1);
        } else if(!DateUtil::comparaDatas($dtiniintcontdfs, $dtfinintcontdfs) && !DateUtil::comparaDatas($dtiniintcontdfs, $dtfinintcontdfs, DateUtil::IGUAL)){
            throw new \Exception("A data inicial do Doc. Fiscais Saída deve ser maior que a data final do Doc. Fiscais Saída.", 1);
        } else if(!DateUtil::comparaDatas($dtinicctbl, $dtfinacctbl) && !DateUtil::comparaDatas($dtinicctbl, $dtfinacctbl, DateUtil::IGUAL)){
            throw new \Exception("A data inicial do Financeiro deve ser maior que a data final do Financeiro.", 1);
        } else if(!DateUtil::comparaDatas($dtiniintcontgpa, $dtfinintcontgpa) && !DateUtil::comparaDatas($dtiniintcontgpa, $dtfinintcontgpa, DateUtil::IGUAL)){
            throw new \Exception("A data inicial da Gestão de Ativos deve ser maior que a data final da Gestão de Ativos.", 1);
        }

        if($this->verificaDatasEntreIntervalo($dtiniccont, $dtfinacont, $dtiniclanc)){
            throw new \Exception("Data inicial do Período de Lançamento deve estar entre o Período Contábil.", 1);
        } else if($this->verificaDatasEntreIntervalo($dtiniccont, $dtfinacont, $dtfinalanc)){
            throw new \Exception("Data final do Período de Lançamento deve estar entre o Período Contábil.", 1);
        } else if($this->verificaDatasEntreIntervalo($dtiniccont, $dtfinacont, $dtiniintcontdfe)){
            throw new \Exception("Data inicial do Doc.Fiscais Entrada deve estar dentro do Período Contábil", 1);
        }else if($this->verificaDatasEntreIntervalo($dtiniccont, $dtfinacont, $dtfinintcontdfe)){
            throw new \Exception("Data final do Doc.Fiscais Entrada deve estar dentro do Período Contábil", 1);
        }else if($this->verificaDatasEntreIntervalo($dtiniccont, $dtfinacont, $dtiniintcontdfs)){
            throw new \Exception("Data inicial do Doc. Fiscais Saída deve estar dentro do Período Contábil", 1);
        }else if($this->verificaDatasEntreIntervalo($dtiniccont, $dtfinacont, $dtfinintcontdfs)){
            throw new \Exception("Data final do Doc. Fiscais Saída deve estar dentro do Período Contábil", 1);
        }else if($this->verificaDatasEntreIntervalo($dtiniccont, $dtfinacont, $dtinicctbl)){
            throw new \Exception("Data inicial do Financeiro deve estar dentro do Período Contábil", 1);
        }else if($this->verificaDatasEntreIntervalo($dtiniccont, $dtfinacont, $dtfinacctbl)){
            throw new \Exception("Data final do Financeiro deve estar dentro do Período Contábil", 1);
        }else if($this->verificaDatasEntreIntervalo($dtiniccont, $dtfinacont, $dtiniintcontgpa)){
            throw new \Exception("Data inicial da Gestão de Ativos deve estar dentro do Período Contábil", 1);
        }else if($this->verificaDatasEntreIntervalo($dtiniccont, $dtfinacont, $dtfinintcontgpa)){
            throw new \Exception("Data final da Gestão de Ativos deve estar dentro do Período Contábil", 1);
        }
    }

    public function verificaData($dataBanco, $dataRequest){
        if ($dataBanco) {
            $dataBanco = DateUtil::getDataDeString($dataBanco[0]["MAIORDATA"], DateUtil::FORMATO_DATAHORA);
            $dataRequest = DateUtil::getDataDeString($dataRequest.'00:00:00', DateUtil::FORMATO_DATAHORA);
            $aux = DateUtil::comparaDatas($dataBanco, $dataRequest, DateUtil::IGUAL);
            if(!$aux){
                return DateUtil::comparaDatas($dataRequest, $dataBanco, DateUtil::ANTES);
            } else {
                return true;
            }
        } else {
            return true;
        }
    }

    public function verificaDatasEntreIntervalo($dataInical, $dataFinal, $dataVerificada){
        $datas = DateUtil::getDatasEntre($dataInical, $dataFinal);
        for ($i=0; $i < count($datas) ; $i++) {
            if(DateUtil::comparaDatas($datas[$i], $dataVerificada, DateUtil::IGUAL)){
                return false;
            }
        }
        return true;
    }

    public function requestSet($row, $dataSourceName, $method, $routePath, $userId){
        $dataSet = new DataSet($dataSourceName, $row);
        $request = new DTO\Request\DataSet($dataSet, $method, $routePath, $userId);
        return $request;
    }
}